use std::time::Duration;

use aspotify::{Client, ClientCredentials};

pub enum SpotifyKind {
    Track(String),
    Album(String),
}

fn get_id_in_url(url: &str) -> Option<&str> {
    url.rsplit('/')
        .next()
        .and_then(|x| x.split(' ').next())
        .and_then(|x| x.split('?').next())
}

pub fn get_spotify_entry(url: &str) -> Option<SpotifyKind> {
    if url.contains("https://open.spotify.com/track/") {
        let track_id = get_id_in_url(url);
        return match track_id {
            Some(id) => Some(SpotifyKind::Track(id.to_string())),
            None => None,
        };
    }
    if url.contains("https://open.spotify.com/album/") {
        let album_id = get_id_in_url(url);
        return match album_id {
            Some(id) => Some(SpotifyKind::Album(id.to_string())),
            None => None,
        };
    }
    return None;
}

pub fn get_spotify_client() -> Box<Client> {
    let spotify_creds =
        ClientCredentials::from_env().expect("CLIENT_ID and CLIENT_SECRET not found.");
    let spotify_client = Box::new(Client::new(spotify_creds));
    spotify_client
}

struct TrackInfo {
    name: String,
    artists: Vec<String>,
    duration: Duration,
}

struct AlbumInfo {
    pub(crate) name: String,
    pub(crate) artists: Vec<String>,
    pub(crate) genres: Vec<String>,
    songs: Vec<TrackInfo>,
}

pub async fn get_spotify_track(spotify: Box<Client>, id: &String) -> Option<TrackInfo> {
    match spotify.tracks().get_track(id.as_str(), None).await {
        Ok(track) => Some(TrackInfo {
            name: track.data.name,
            artists: track.data.artists.iter().map(|x| x.name.clone()).collect(),
            duration: track.data.duration,
        }),
        Err(_e) => None,
    }
}

pub async fn get_spotify_album(spotify: Box<Client>, id: &String) -> Option<AlbumInfo> {
    match spotify.albums().get_album(id.as_str(), None).await {
        Ok(album) => Some(AlbumInfo {
            name: album.data.name,
            artists: album.data.artists.iter().map(|x| x.name.clone()).collect(),
            genres: album.data.genres,
            songs: vec![], // TODO we could lookup songs and put them here!
        }),
        Err(_e) => None,
    }
}
